#!/usr/bin/env python
from gimpfu import *

'''
 BA_PlasmaticV1c http://gimpchat.com/viewtopic.php?f=11&t=15362&hilit=plasmatic
 A sorry attempt to duplicate Lylejk Plasma workflow on Gimp Chat,
 having limited success.
 
*** Script requires G'MIC for GIMP version 1.79 ****

 
 A color or black and white subject which has been extracted 
 from its background has advantage.  Regular image will work.
 Sometimes nice to use brightness/contrast to lighten subject.
 
 With an extracted subject, first hoose a solidify option.  
 It will fill the transparent areas, different options will
 give variations.  You may select to desaturate the both the
 subject and highlight layers or either of them, or not.
 
 Choose a scale percentage and the program will scale the original
 up or down to start with.  The outcome will be returned to the 
 original dimensions.  The process will create  Highlight result layer
 and then will scale the image up and create an 'Object' result layer
 
 The plasma lines may be controlled with the Noise Spread parameters
 (one each for Highlight and Object).
 Same with the length with Segmentation parameters.
 The Line Width may be decreased with the width parameter.
 A note: should your Highlight layer or the Subject layer outcome
 be all or mostly black perhaps with some artifacts, the  Line Width
 setting should be decreased. 
 
 There is a control for upscale 1x or 2x (each upscale is approx 150 percent),
 before creating the subject layer, and when the outcome is scaled
 back to the original size, it will 'tighten' the pattern.
 
 If you have an 'extracted' image with a transparent background,
 the Enhance BG toggle will extract the subject from the layer and
 intensify it, producing an outcome layer set to 5% opacity which may be raised 
 to your taste to brighten the 'background'  
 It is nice to use the color select tool on the color between the lines
 and then delete it. The lines will stand out more.
 
 There is also a subject layer intensity which will affect the subject layer only.
 If you would like the  highlight layer more intense, check that the layer is 
 set to SCREEN mode and duplicate it.  Again if the lines are not clear or the
 layer is all black the Line With decrease needs to be moved lower.
 
 There is a Map Color gradient option to try.
 You can choose to have the original subject properly scaled and positioned on
 on the top of the stack.
 
 The purpose of this script is to make layers quickly so that you may
 use 'Layer Play' to finesse your final outcome.
 
 When learning this flow, it may be best to start with 
 smaller subjects maybe 800 to 1900 in width. Some of the
 G'MIC filters take a few minutes to run.
 
 I make a 1st run, quickly inspect the subject layers for focus while
 turning them off to look at the highlight (bottom 2) layers to verify
 the line width is good and in 'focus', then make adjustments and run again.
 When the highlights are fine I the same for the object layer.
 
 Some times I will use an extracted subject, add a transparent layer below it,
 add a color and noise to the transparent layer, then use blur on it.  Then
 merge it with the subject and run the filter with None for solidify type. 
 
 Hope that you enjoy it!
 Brian Andersen
 animicule@gimpchat.com

...................................................... 
####################
 COPYRIGHT NOTICE 
 You can view the GNU General Public  License version 3 
 at the web site http://www.gnu.org/licenses/gpl-3.0.html
####################
'''
	
def BA_PlasmaticR1(inImage, inDrawable, SFlag, scalepct, desat, sq1ns, sq1seg, himorph1, sq2ns, sq2seg, subjmorph, upscalex, inTens, enHance, cmgrad, fgsel, bgsel, addsub,brightness, contrast ):

    pdb.gimp_context_push
    currFG = pdb.gimp_context_get_foreground()
    currBG = pdb.gimp_context_get_background()

    width  = inImage.width
    height = inImage.height
    type   = inImage.base_type
    
    Scale  = scalepct/100

    #build option string for filename copy to export to keep record of settings
    #set toggle vars to No and test for Yes
    P1=P8=P11=P12=P13=P14=P15='N'
    skipit = False
    if SFlag   == 0:
        P2 = "None"
        skipit = True
    elif SFlag == 1:
        P2 = "iso1"
    elif SFlag == 2:
        P2 = "del1"
    elif SFlag == 3:
        P2 = "edg1"  
    elif SFlag == 4:
        P2 = "iso2"       
    elif SFlag == 5:
        P2 = "del2"
    elif SFlag == 6:
        P2 = "edg2"   
    elif SFlag == 7:
        P2 = "iso3"  
    elif SFlag == 8:
        P2 = "del4"    
    elif SFlag == 9:
        P2 = "iso5"
    P3  = str(scalepct)
    P4  = str(desat)
    #init desats to N and test for Y
    desat1 = desat2 = False
    if desat  == 0:
        None
    elif desat == 1:
        desat1 = True
        desat2 = True
    elif desat == 2:
        desat1 = True
    elif desat == 3:
        desat2 = True
    P5  = str(sq1ns)
    P6  = str(sq1seg) 
    P7  = str(himorph1) 
    P8  = str(sq2ns)
    P9  = str(sq2seg) 
    P10 = str(inTens)
    P11 = str(subjmorph)
    if upscalex:
        P12 = "Y"
    if enHance:
        P13 = "Y"
    if cmgrad:
        P14 = "Y"
    if addsub:
        P15 = "Y"

    name = inDrawable.name
    if '.' in name:
        split = name.split(".", 1)
        name = split[0]
    suffix ="_BA_PlasmaticR1"+P1+","+P2+","+P3+","+P4+","+P5+","+P6+","+P7+","+P8+","+P9+","+P10+","+P11+","+P12+","+P13+","+P14+","+P15
    
    newwidth=int (width *Scale)
    newheight=int(height*Scale)

    newImage = pdb.gimp_image_new (width, height, 0)     
    savBG = pdb.gimp_layer_new_from_drawable(inDrawable, newImage)     #save subject layer

    savBG.mode = LAYER_MODE_NORMAL_LEGACY
    savBG.name = "Save for Use"
    savBG.opacity = 100.0 
#   newImage.add_layer (savBG, type)
    newImage.add_layer (savBG, 0)	# MrQ type 0=RGB
    pdb.gimp_item_set_visible(savBG, False)    
    alpha=pdb.gimp_drawable_has_alpha(savBG)
    if (alpha==False):
        pdb.gimp_layer_add_alpha (savBG)    #  RGBA

    newBG = savBG.copy()
    newBG.mode = LAYER_MODE_NORMAL_LEGACY
    newBG.name = "Extracted Subject" 
    newBG.opacity = 100.0 
    newImage.add_layer (newBG, 0)
    pdb.gimp_item_set_visible(newBG, True)    
    
    if not skipit:
        if SFlag   ==  1:   # solidify options - more web with type1 type2 
            pdb.plug_in_gmic_qt(newImage, newBG, 1,0, "-v - -fx_solidify_td 70,0,0,-3,0")    #iso 1
        elif SFlag ==  2:
            pdb.plug_in_gmic_qt(newImage, newBG, 1,0, "-v - -fx_solidify_td 70,1,0,-3,0")    #del 1
        elif SFlag ==  3:
            pdb.plug_in_gmic_qt(newImage, newBG, 1,0, "-v - -fx_solidify_td 70,2,0,-3,0")    #edg 1
        elif SFlag ==  4:  
            pdb.plug_in_gmic_qt(newImage, newBG, 1,0, "-v - -fx_solidify_td 70,0,100,-3,0")  #iso 2
        elif SFlag ==  5:
            pdb.plug_in_gmic_qt(newImage, newBG, 1,0, "-v - -fx_solidify_td 70,1,100,-3,0")  #del 2
        elif SFlag ==  6:
            pdb.plug_in_gmic_qt(newImage, newBG, 1,0, "-v - -fx_solidify_td 70,2,100,-3,0")  #edg 2
        elif SFlag ==  7:     
            pdb.plug_in_gmic_qt(newImage, newBG, 1,0, "-v - -fx_solidify_td 20,0,100,-1,0")  #iso 3 
        elif SFlag ==  8:
            pdb.plug_in_gmic_qt(newImage, newBG, 1,0, "-v - -fx_solidify_td 35,1,20,0,0")    #del 4
        elif SFlag ==  9:
            pdb.plug_in_gmic_qt(newImage, newBG, 1,0, "-v - -fx_solidify_td 50,0,0,4,0")     #iso 5  
    
    pdb.gimp_image_scale(newImage, newwidth, newheight) #interpolation LANCZOS=3)
    pdb.gimp_item_set_visible(newBG, False)
    newBG.name = "Save for Later Use"
    oneLay = newBG.copy()
    oneLay.mode = LAYER_MODE_NORMAL_LEGACY
    oneLay.name = "oneLay"
    oneLay.opacity = 100.0 
    newImage.add_layer(oneLay, 0)
    pdb.gimp_item_set_visible(oneLay, False)
    
    #[G'MIC] Make squiggly: -fx_Squiggly 7,12,4.27,0,0.5,0,0,0,3,0.2,0.4,12,1,1,0    
    pdb.plug_in_gmic_qt(newImage, oneLay, 1,0,    # 1=input only active layer
    "-v - -fx_Squiggly "+str(sq1ns)+","+str(sq1seg)+",4.27,0,0.5,0,0,0,3,0.2,0.4,12,1,1,0")
    oneLay.name = "small squiggly"
    pdb.gimp_item_set_visible (oneLay, False)
   
    #run 2x [G'MIC] Upscale [dcci2x]: -gimp_scale_dcci2x 1.15,5,0
    pdb.plug_in_gmic_qt(newImage, oneLay, 1,0,    # 1=input only active layer
    "-v - -fx_scale_dcci2x 1.15,5,0")
    bigLay = pdb.gimp_image_get_active_layer(newImage)         
    bigLay.name = 'upscaled 1x'

    if upscalex:
        pdb.plug_in_gmic_qt(newImage, bigLay, 1,0,    # 1=input only active layer
        "-v - -fx_scale_dcci2x 1.15,5,0")
        bigLay = pdb.gimp_image_get_active_layer(newImage) 
        bigLay.name = 'upscaled 2x'
        pdb.gimp_item_set_visible (bigLay, False)  

    #[G'MIC] Morphological filter: -gimp_morpho_v2 0,0,INTorFLOAT,"1,0,1; 0,1,0; 1,0,1",0,0,0,1,0
    pdb.plug_in_gmic_qt(newImage, bigLay, 1,0,    # 1=input only active layer
    '-v - -fx_morphological 0,0,'+str(himorph1)+','+'"1,0,1; 0,1,0; 1,0,1",0,0,0,1,0') ## was 2, trying to make highlight thinner
    highlightLay = pdb.gimp_image_get_active_layer(newImage)
    highlightLay.name = name+suffix
    
    if desat1:
        pdb.gimp_drawable_desaturate(highlightLay, 1)
    pdb.gimp_item_set_visible(highlightLay,  False) 
    
    iCnt=0
    while (iCnt < inTens):
        TMPLay1 = highlightLay.copy()
        TMPLay1.opacity = 100.0 
        TMPLay1.mode = LAYER_MODE_SCREEN_LEGACY
        iCnt+=1         
        TMPLay1.name = 'LoopTMP1'+str(iCnt)
        newImage.add_layer(TMPLay1, 0)
        pdb.gimp_item_set_visible(TMPLay1, True)
    highlight2Lay = pdb.gimp_image_merge_visible_layers(newImage, 0)
    highlight2Lay.opacity = 50.0     
    highlight2Lay.name = 'Highlight Intensity'
    highlight2Lay.mode = LAYER_MODE_SCREEN_LEGACY
    pdb.gimp_item_set_visible(highlight2Lay, False)

    mwidth  = pdb.gimp_drawable_width (highlightLay)
    mheight = pdb.gimp_drawable_height(highlightLay)
    pdb.gimp_layer_scale(newBG, mwidth, mheight, True)
    pdb.gimp_image_raise_item_to_top(newImage, newBG)
    newBG.name = 'scaled'
    pdb.gimp_layer_set_offsets(newBG,0,0)    
    pdb.gimp_item_set_visible(newBG, False)
    if desat2:
        pdb.gimp_drawable_desaturate(newBG, 1)

    pdb.gimp_image_set_active_layer(newImage, newBG)
    pdb.plug_in_gmic_qt(newImage, newBG, 1,0,    # 1=input only active layer
     "-v - -fx_Squiggly "+str(sq2ns)+","+str(sq2seg)+",1.8,0,0.5,0,0,0,3,0.2,0.4,12,1,1,0")  

    topLay = newBG.copy()
    topLay.mode = LAYER_MODE_SCREEN_LEGACY 
    topLay.name = 'Object Layer'
    topLay.opacity = 100.0
    newImage.add_layer(topLay, 0)
    pdb.gimp_item_set_visible(topLay,False)    

    newBG.name = "Object one"
    newBG.mode = LAYER_MODE_SCREEN_LEGACY
    pdb.gimp_item_set_visible(newBG, True)
    iCnt=0
    while (iCnt < inTens):
        TMPLay = newBG.copy()
        TMPLay.opacity = 100.0 
        TMPLay.mode = LAYER_MODE_SCREEN_LEGACY
        iCnt+=1         
        TMPLay.name = 'LoopTMP2'+str(iCnt)
        newImage.add_layer(TMPLay, 0)
        pdb.gimp_item_set_visible(TMPLay, True)
    newBG = pdb.gimp_image_merge_visible_layers(newImage, 0)
    newBG.name = 'Object Intensity'
    newBG.opacity = 5.0
    newBG.mode = LAYER_MODE_SCREEN_LEGACY
    pdb.gimp_item_set_visible(newBG, False)
    
    #[G'MIC] Morphological filter: 
    pdb.plug_in_gmic_qt(newImage, newBG, 1,0,    # 1=input only active layer
    '-v - -fx_morphological 0,0,'+str(subjmorph)+','+'"1,0,1; 0,1,0; 1,0,1",0,0,0,1,0') ## was 2, trying to make highlight thinner    

    if enHance:
        cutLay = newBG.copy()
        cutLay.mode = LAYER_MODE_NORMAL_LEGACY
        cutLay.name = "cutLay" 
        cutLay.opacity = 100.0 
        newImage.add_layer (cutLay, 0)
        pdb.gimp_item_set_visible(cutLay, True)

        xtrkdTMP = savBG.copy()
        xtrkdTMP.mode = LAYER_MODE_NORMAL_LEGACY
        xtrkdTMP.name = "xtrkdTMP extracted subject" 
        xtrkdTMP.opacity = 100.0 
        newImage.add_layer (xtrkdTMP, 0)
        pdb.gimp_item_set_visible(xtrkdTMP, True)    
        pdb.gimp_layer_scale(xtrkdTMP, mwidth, mheight, True)  # pdb.gimp_layer_scale(layer ,new-width ,new-height ,local-origin ,interpolation)
		                                                               #      pdb.gimp_layer_scale(layer ,new-width ,new-height ,local-origin)
        pdb.gimp_layer_set_offsets(xtrkdTMP,0,0)
        if desat2:
            pdb.gimp_drawable_desaturate(xtrkdTMP, 1)

        pdb.gimp_image_select_item(newImage, 0, xtrkdTMP)
        pdb.gimp_selection_shrink(newImage, 1)
        pdb.gimp_edit_cut(cutLay)
        pdb.gimp_selection_none(newImage)  #deselects the selection
        pdb.gimp_image_remove_layer(newImage, xtrkdTMP) #remove the temp file

        iCnt=0
        while (iCnt < inTens):
            TMPLay = cutLay.copy()
            TMPLay.opacity = 100.0 
            TMPLay.mode = LAYER_MODE_SCREEN_LEGACY
            iCnt+=1        
            TMPLay.name = 'LoopTMP3'+str(iCnt)
            newImage.add_layer(TMPLay, 0)
            pdb.gimp_item_set_visible(TMPLay, True)
        pdb.gimp_item_set_visible(cutLay, False)
        enhanceBG = pdb.gimp_image_merge_visible_layers(newImage, 0)
        enhanceBG.name = 'Enhanced BG'
        enhanceBG.opacity = 5.0
        enhanceBG.mode = LAYER_MODE_SCREEN_LEGACY
        pdb.gimp_item_set_visible(enhanceBG, False)
        pdb.gimp_image_remove_layer(newImage, cutLay) #remove the temp file        

    if cmgrad:   ## color map gradient - may not be implemented correctly ??
        cmLay = newBG.copy()
        newImage.add_layer(cmLay, 4)
        pdb.gimp_item_set_visible(cmLay, False)
        pdb.gimp_context_set_foreground(fgsel)
        pdb.gimp_context_set_background(bgsel)
        pdb.plug_in_gradmap(newImage, cmLay)
        cmLay.mode = 5 
        cmLay.opacity = 100.0
        cmLay.name = 'gradient map overlay' 
        cmLay1 = cmLay.copy()
        cmLay1.mode = 13
        cmLay1.opacity = 50.0
        cmLay1.name = 'COLOR 50%'                   
        newImage.add_layer(cmLay1, 5)     
        cmLay.name = "Gradient Map Overlay"
        cmLay.mode = 5

    padLayer = newBG.copy()       
    newImage.add_layer(padLayer, 0)  
    pdb.gimp_image_raise_item_to_top(newImage, padLayer)   
    padLayer.mode = LAYER_MODE_NORMAL_LEGACY
    padLayer.opacity = 100.00
    padLayer.name = "Pad Color"
    pdb.gimp_context_set_foreground('#000000')
    pdb.gimp_drawable_fill(padLayer, 0)
    pdb.gimp_item_set_visible(padLayer,True) 
#   
#######  layer creation done, organize outcome
#
    pdb.gimp_item_set_visible(highlightLay, True) # Named Layer - Screen 50.0
    pdb.gimp_image_raise_item_to_top(newImage, highlightLay)
    highlightLay.mode = LAYER_MODE_SCREEN_LEGACY
    highlightLay.opacity = 100.0
    pdb.gimp_item_set_visible(highlight2Lay,False) # Highlight Intensity
    pdb.gimp_image_raise_item_to_top(newImage, highlight2Lay)
    highlight2Lay.opacity = 5.0
    if cmgrad:
        pdb.gimp_image_raise_item_to_top(newImage, cmLay)   #Color 50% - name and settings
        pdb.gimp_image_raise_item_to_top(newImage, cmLay1)  #Gradient Map - overlay 100.0
        pdb.gimp_item_set_visible(cmLay, True)
        pdb.gimp_item_set_visible(cmLay1,True)
    if enHance:
        pdb.gimp_image_raise_item_to_top(newImage, enhanceBG) #Enhanced BG
        enhanceBG.opacity = 5.0
        pdb.gimp_item_set_visible(enhanceBG, False)
    pdb.gimp_image_raise_item_to_top(newImage, newBG)  #Object Intensity
    newBG.mode = LAYER_MODE_SCREEN_LEGACY
    newBG.opacity = 5.0
    pdb.gimp_item_set_visible(newBG, False)
    # object layer to position 1
    pdb.gimp_image_raise_item_to_top(newImage, topLay)  #Object Layer
    pdb.gimp_item_set_visible(topLay, True)
    #pdb.gimp_drawable_brightness_contrast(topLay, 0., 0.4)

# add MrQ ##################################################################		
    num_layers, layer_ids = pdb.gimp_image_get_layers(newImage)
    layer_names_list = newImage.layers		
    for layer in newImage.layers:
        pdb.gimp_image_set_active_layer(newImage,layer)
        pdb.gimp_item_set_visible(layer, True)
        pdb.gimp_item_set_visible(savBG, False)
        if layer.visible:
            pdb.gimp_drawable_levels_stretch(layer)		
            pdb.gimp_drawable_brightness_contrast(layer, brightness, contrast)

# ############################################################################		
    if addsub: # original subject to position 0 and resized
        pdb.gimp_image_raise_item_to_top(newImage, savBG)
        savBG.mode = LAYER_MODE_NORMAL_LEGACY
        savBG.name = "Original Subject" 
        savBG.opacity = 35.0 
        pdb.gimp_layer_scale(savBG, mwidth, mheight, True)
        pdb.gimp_layer_set_offsets(savBG,0,0) 
        pdb.gimp_item_set_visible(savBG, False)
    else: #remove unused layer
        pdb.gimp_image_remove_layer(newImage, savBG)
		
     # scale back to original size
    pdb.gimp_image_scale(newImage, width, height)

    # Clean Up
    pdb.gimp_display_new(newImage)
    pdb.gimp_displays_flush
   
    # Close the undo group.
    pdb.gimp_context_pop
    pdb.gimp_context_set_foreground(currFG)
    pdb.gimp_context_set_background(currBG)
    
    return(BA_PlasmaticR1)  

# This is the plugin registration function
register(
    "BA_PlasmaticV1c",    
    "May use extracted subject or not. Requires G'mic.",   
    "May use extracted subject or not",
    "Brian Andersen", 
    "animicule@gimpchat.com", 
    "December 31, 2016",
    "BA Plasmatic...", 
    "*",                    
     
    [
      (PF_IMAGE,      "image"    , "Input image"                           ,  None),
      (PF_DRAWABLE,   "idrawable", "Input drawable"                        ,  None),
      (PF_OPTION,     "sflag"    , "Solidify type or none if not extracted",  5,   ("None","iso1","del1","edg1","iso2","del2","edg2","iso3","del4","iso5",)),
      (PF_ADJUSTMENT, "scalepct" , "Scale Percentage"                      , 40,   (10,300, 1)),
      (PF_OPTION,     "desat"    , "Desaturate Options: No,HL,Sub,Both"    ,  0,   ("None","Both","Highlight Only","Subject Only",)),
      (PF_SPINNER,     "sq1ns"    , "Highlight Noise Spread (like a focus)_",  2,   ( 0.0, 20.0, 0.1)),
      (PF_SPINNER,     "sq1seg"   , "Highlight Segmentation length"         ,4.8,   ( 0.5, 15.0, 0.1)),
      (PF_SPINNER,     "himorph1" , "Highlight Width decrease"              ,  7,   (   0,  10,   1  )),
      (PF_SPINNER,     "sq2ns"    , "Object Layer NoiseSpread (like focus)" ,  7,   ( 0.0, 20.0, 0.1)),      
      (PF_SPINNER,     "sq2seg"   , "Object Layer Segmentation"             , 10,   ( 0.5, 15.0, 0.1)),
      (PF_SPINNER,     "subjmorph", "Object Layer Line Width decrease"      ,  2,   (   0,  7,   1  )),
      (PF_TOGGLE,     "upscale2x", "Upscale Twice(Yes) or Once if No"      ,  True),
      (PF_SPINNER,     "inTens"   , "Intensity for Hlight BG and Object"    , 10,   (   4, 20,   1  )),
      (PF_TOGGLE,     "enHance"  , "Enhance BG (Only works if extracted)"  , False),
      (PF_TOGGLE,     "cmgrad"   , "Map ColorGradient? (choose colors)"    , False),
      (PF_COLOR,      "fgsel"    , "FG Color for color map"                , (1.0, 1.0, 1.0)),      
      (PF_COLOR,      "bgsel"    , "BG Color for color map"                , (1.0, 1.0, 1.0)),
      (PF_TOGGLE,     "addsub"   , "Add Subject to Top"                    , False),
      (PF_SPINNER,     "brightness"    , "Brightness",  0.45,   ( -0.5, 0.5, 0.01)),	  
       (PF_SPINNER,     "contrast"    , "Contrast",  0.40,   ( -0.5, 0.5, 0.01)),     
    ], 
    [
      (PF_DRAWABLE, "odrawable", "Output drawable", None),  
    ],
    BA_PlasmaticR1,
    menu="<Image>/Filters/Artistic/Brian/",
    )

main()
